import time
import math
import os
import datetime

from generic_module import Module
from collections import deque
from lxml import etree
import threading

class AggregationModule(Module):
  def __init__(self, name, matching_timeout, fuzziness, save_to_disk, boss_receive, root_path):
    Module.__init__(self, name, root_path)
    self.matching_timeout = matching_timeout
    self.fuzziness = fuzziness
    self.save_to_disk = save_to_disk
    self.timeout = 0
    self.boss_receive = boss_receive
    
    self.vehicles_dict = dict()
    self.deque_lock = threading.Lock()
    
  def add_vehicle(self, vehicle, recv_module_name):
    sub_vehicle = vehicle.getchildren()[0]
    #print etree.tostring(sub_vehicle)
    if recv_module_name not in self.vehicles_dict:
      self.vehicles_dict[recv_module_name] = deque()
    with self.deque_lock:
      self.vehicles_dict[recv_module_name].append(sub_vehicle)
  
  #--- "private"
  def find_and_form(self, wim):
    ts = wim.find("ts").text
    start = time.mktime(datetime.datetime.strptime(ts, "%Y-%m-%d-%H-%M-%S-%f").timetuple())
    #---
    vehicle = etree.Element("vehicle")
    vehicle.append(wim)
    #clear "dead" recvs
    for key in self.vehicles_dict:
      if key == self.boss_receive:
        continue
      try:
        with self.deque_lock:
          while len(self.vehicles_dict[key]) > 0 and self.compare_timestamps(self.vehicles_dict[key][0].find("ts").text, wim.find("ts").text) == 3:
            self.vehicles_dict[key].popleft()
      except Exception:
        self.log("Find and form error 1.")
        try:
          self.deque_lock.release()
        except Exception:
          pass
        return
    found_list = list()
    found_list.append(self.boss_receive)
    #---
    while True:
      if len(found_list) == len(self.vehicles_dict):
        #broken = True
        break
      for key, value in self.vehicles_dict.iteritems():
        if key not in found_list:
          try:
            with self.deque_lock:
              for veh_child in value:
                if self.compare_timestamps(wim.find("ts").text, veh_child.find("ts").text) == 1:
                  vehicle.append(veh_child)
                  found_list.append(key)
                  #broken = True
                  break
          except Exception:
            self.log("Find and form error 2.")
            try:
              self.deque_lock.release()
            except Exception:
              pass
            break
      if time.time() - start > self.matching_timeout:
        break
      time.sleep(0.5)
    return vehicle   
    
  def save_to_file(self, str_vehicle):
    #print "start save"
    if self.sname == "":
      return 1
    first_line = "<swd><site><name>" + self.sname + "</name><vehicles>\n"
    last_line = "</vehicles></site></swd>"
    try:
      date = datetime.datetime.now().strftime('%Y-%m-%d')
      if not os.path.exists(self.sites_path + "/" + self.sname + "/ext"):
        os.mkdir(self.sites_path + "/" + self.sname + "/ext")
      file = open(self.sites_path + "/" + self.sname + "/ext/" + date + ".xml", "a+")
      #remove last line          
      file.seek(0, os.SEEK_END)
      pos = file.tell() - 1
      while pos > 0 and file.read(1) != "\n":
        pos -= 1
        file.seek(pos, os.SEEK_SET)
      if pos > 0:
        file.seek(pos, os.SEEK_SET)
        file.truncate()
      #if file is empty, add first line
      if self.sname != "" and pos == file.tell() - 1:
        file.write(first_line + str_vehicle + "\n" + last_line)
      else:
        file.write(str_vehicle + "\n" + last_line)
      file.close()
    except Exception, ex:
      try:
        file.close()
      except Exception:
        pass
      self.log(str(ex))
    #print "end save"
        
  def compare_timestamps(self, ts1, ts2):
    parts1 = ts1.split("-")
    parts2 = ts2.split("-")
    
    timestamp1 = time.mktime(datetime.datetime.strptime(ts1, "%Y-%m-%d-%H-%M-%S-%f").timetuple()) + float(parts1[len(parts1) - 1]) / 1000
    timestamp2 = time.mktime(datetime.datetime.strptime(ts2, "%Y-%m-%d-%H-%M-%S-%f").timetuple()) + float(parts2[len(parts2) - 1]) / 1000
    
    if math.fabs(timestamp1 - timestamp2) <= self.fuzziness:
      return 1
    else:
      if timestamp1 > timestamp2:
        return 2
      else:
        return 3
            
  def run(self):
    self.alive = True
    self.end = False
    #self.log("Starting " + self.name + " aggregation module ...")
    try:
      while True:
        if self.end:
          self.alive = False
          return   
        if self.boss_receive in self.vehicles_dict and len(self.vehicles_dict[self.boss_receive]) > 0:
          wim = self.vehicles_dict[self.boss_receive].popleft()
          vehicle = self.find_and_form(wim)
          if self.save_to_disk == 1:
            vehicle_str = etree.tostring(vehicle)
            self.save_to_file(vehicle_str)
          #print etree.tostring(vehicle)
          for key, mod in self.downstream_modules_dict.iteritems():
            mod.add_vehicle(vehicle)
            mod.set_site_name(self.sname)
          #print etree.tostring(vehicle)
        time.sleep(0.5)
    except Exception, ex:
      self.log(str(ex))
      self.alive = False
