from __future__ import division
import sys
import os
import re
import time
from sys import platform
import datetime

from generic_module import Module

if platform == "win32":
    from ctypes import c_ulonglong, windll, c_wchar_p, pointer
elif platform == "linux" or platform == "linux2":
    import subprocess


class ClearModule(Module):

    def __init__(self, name, root_dir, refresh_timer, minimum_disk_space_threshold, maximum_disk_space_threshold, priority_list, root_path):
        Module.__init__(self, name, root_path)
        self.name = name
        self.root_dir = root_dir
        self.refresh_timer = refresh_timer
        self.minimum_disk_space_threshold = minimum_disk_space_threshold
        self.maximum_disk_space_threshold = maximum_disk_space_threshold
        self.priority_list = priority_list

    def w32DiskFreeMB(self, p):
        lpFreeBytesAvailable = c_ulonglong(0)
        lpTotalNumberOfBytes = c_ulonglong(0)
        lpTotalNumberOfFreeBytes = c_ulonglong(0)
        windll.kernel32.GetDiskFreeSpaceExW(c_wchar_p(p), pointer(lpFreeBytesAvailable), pointer(lpTotalNumberOfBytes),
                                            pointer(lpTotalNumberOfFreeBytes))
        return lpTotalNumberOfFreeBytes.value / 1024 / 1024

    def uniX32PartitionFreeMB(self, mp):
        sp = subprocess.Popen(["df", "-B", "MB", mp], stdout=subprocess.PIPE)
        output = sp.communicate()[0]
        size = output.split("\n")[1].split()[3]
        return int(size[:size.find("M")])

    def get_free_disk_space(self):
        # gets current_disk_space
        current_disk_space = sys.maxint
        if platform == "win32":
            current_disk_space = self.w32DiskFreeMB("c:") / 2 ** 10
        elif platform == "linux" or platform == "linux2":
            current_disk_space = self.uniX32PartitionFreeMB("/siwim") / 2 ** 10
        return current_disk_space

    # returns a list of all directories/files in "dir_path" directory, sorted by modification time
    def get_dir_list(self, dir_path):
        files = []
        for s in os.listdir(dir_path):
            if os.path.isdir(os.path.join(dir_path, s)) or not datetime.datetime.fromtimestamp(os.path.getmtime(os.path.join(dir_path, s))).date() == datetime.date.today() or not s.endswith(".log"):
                files.append(str(s))
        files.sort(key=lambda s: os.path.getmtime(os.path.join(dir_path, s)))
        return files

    # returns a list of all directories in "curr_dir" sorted by modification time
    def get_sites(self, curr_dir):
        sites = []
        for s in os.listdir(curr_dir):
            if os.path.isdir(os.path.join(curr_dir, s)):
                sites.append(str(s))
        sites.sort(key=lambda s: os.path.getmtime(os.path.join(curr_dir, s)))
        return sites

    # deletes files that haven't been modified recently first
    def delete_oldest_files(self, subdir_path, diff):

        if not os.path.exists(subdir_path):
            self.log(subdir_path + " doesn't exist")
            return diff

        dir_list = self.get_dir_list(subdir_path)

        for dir in dir_list:

            if diff <= 0:
                return diff

            file_path = os.path.join(subdir_path, str(dir))
            if os.path.isdir(file_path):
                diff = self.delete_oldest_files(file_path, diff)
            elif os.path.isfile(file_path) and not re.match('^[0-9]{4}-[0-9]{2}-[0-9]{2}( [(][0-9]*[)])?.log', dir):
                file_size = os.path.getsize(file_path) / 2 ** 30
                # print "removing: " + file_path + " size: " + str(file_size)
                os.remove(file_path)
                diff -= file_size

        return diff

    def run(self):
        # main loop

        num_of_sites = 1
        select_site = False
        default_dir = self.root_dir
        if self.root_dir.endswith("sites"):
            sites = self.get_sites(self.root_dir)
            num_of_sites = len(sites)
            select_site = True

        #print "root_dir: " + str(self.root_dir)
        #print "refresh_timer: " + str(self.refresh_timer)

        while True:

            # print "calculating size...."

            current_disk_space = self.get_free_disk_space()
            diff = self.maximum_disk_space_threshold - current_disk_space

            #print "maximum_disk_space_threshold: " + str(self.maximum_disk_space_threshold) + " GB"
            #print "minimum_disk_space_threshold: " + str(self.minimum_disk_space_threshold) + " GB"
            #print "current_disk_space: " + str(current_disk_space) + " GB"

            if current_disk_space < self.minimum_disk_space_threshold:

                curr_site = 0
                while num_of_sites > 0:
                    if select_site:
                        self.root_dir = os.path.join(default_dir, sites[curr_site])
                        curr_site += 1


                    priority_level = 0
                    while diff > 0 and priority_level < len(self.priority_list):
                        subdir_path = os.path.join(self.root_dir, self.priority_list[priority_level])
                        diff = self.delete_oldest_files(subdir_path, diff)
                        priority_level += 1

                    num_of_sites -= 1

                if diff > 0:
                    self.log("Couldn't delete " + str(diff) + " GB. Add more directories to the dirs [xml conf]")



            #print "waiting for " + str(self.refresh_timer) + " seconds..."
            # time delay before next loop iteration
            time.sleep(float(self.refresh_timer))
