import socket
import datetime
import urllib2
import os

from lxml import etree
from generic_module import Module

class PapagoModule(Module):
  def __init__(self, name, udp_ip, udp_port, device_ip, interval, indexing_start, root_path):
    Module.__init__(self, name, root_path)
    self.udp_ip = udp_ip
    self.udp_port = udp_port
    
    self.interval = interval
    
    self.device_ip = device_ip
    
    self.protocol = "http://"
    self.loc = "/fresh.xml"
    
    self.indexing_start = indexing_start
    
  def generate_path(self):
    pth = self.sites_path
    pth = pth + "/" + self.sname
    if not os.path.exists(pth):
      os.mkdir(pth)
    pth = pth + "/ext"
    if not os.path.exists(pth):
      os.mkdir(pth)
    pth = pth + "/temperature"
    if not os.path.exists(pth):
      os.mkdir(pth)
    #pth = pth + "/papago"
    #if not os.path.exists(pth):
      #os.mkdir(pth)
    """
    pth = pth + "/" + self.typ
    if not os.path.exists(pth):
      os.mkdir(pth)
    pth = pth + "/" + self.name 
    if not os.path.exists(pth):
      os.mkdir(pth)
    """
    #print pth
    return pth
  
  def get_data(self):
    data_url = self.protocol + self.device_ip + self.loc
    #print data_url
    try:
      request = urllib2.urlopen(data_url)
    except Exception, ex:
      self.log(str(ex))
      return "Err"
    xml = request.read()   
    xmlns = xml[xml.find("xmlns=\""):xml.find("\">") + 1]
    xml = xml.replace(xmlns, "")
    
    temp_data = etree.fromstring(xml)
    #print temp_data.attrib["xmlns"]
    
    #td = etree.Element("td")
    #site = etree.Element("site")
    #td.append("site")
    #name = etree.Element("name")
    #name.text = self.sname
    #site.append(name)
    #temperatures = etree.Element("temperature_readings")
    #site.append(temperatures)
    
    temperature = etree.Element("temperature_reading")
    mname = etree.Element("module_name")
    mname.text = self.name
    
    #print etree.tostring(temp_data)
    #ts = temp_data.find("status").attrib["time"]
    our_ts = datetime.datetime.now().strftime('%Y-%m-%d-%H-%M-%S')
    ts_el = etree.Element("ts")
    ts_el.text = our_ts
    temperature.append(ts_el)
    
    values = etree.Element("values")
    temperature.append(values)
    
    sns = temp_data.findall("sns")
    
    curr_idx = self.indexing_start
    
    for sn in sns:
      value = float(sn.attrib["val"])
      status = sn.attrib["status"]
      ut = int(sn.attrib["unit"])
      if status != "0":
        self.log("Status: " + status + ". Consult documentation.")
      #sensor = etree.Element("sensor")
      
      if ut == 1:
        value = (value - 32.0) * (5 / 9) 
      if ut == 2:
        value = value - 273.15
      
      val = etree.Element("value")
      val.text = str(value)
      
      val.attrib["idx"] = str(curr_idx)
      #maybe usable at a later time
      val.attrib["name"] = "some_name"
      val.attrib["unit"] = "C"
      values.append(val)
      curr_idx+=1
    return temperature
  
  def save_to_file(self, data):
    if self.sname == "":
      return
    first_line = "<td><site><name>" + self.sname + "</name><temperature_readings>\n"
    last_line = "</temperature_readings></site></td>"
    try:
      date = datetime.datetime.now().strftime('%Y-%m-%d')
      path = self.generate_path()
      fd = open(path + "/" + date + ".xml", "a+")
      #remove last line          
      fd.seek(0, os.SEEK_END)
      pos = fd.tell() - 1
      while pos > 0 and fd.read(1) != "\n":
        pos -= 1
        fd.seek(pos, os.SEEK_SET)
      if pos > 0:
        fd.seek(pos, os.SEEK_SET)
        fd.truncate()
      #if file is empty, add first line
      if pos == fd.tell() - 1:
        fd.write(first_line + etree.tostring(data) + "\n" + last_line)
      else:
        fd.write(etree.tostring(data) + "\n" + last_line)
      fd.close()
    except Exception, ex:
      try:
        fd.close()
      except Exception:
        pass
      self.log(str(ex))
  
  def send_data(self, data):
    conf_str_data = self.xml_to_conf_str("mydevice", data)
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    try:
      sock.sendto(conf_str_data, (self.udp_ip, self.udp_port))
    finally:
      sock.close()
  
  def xml_to_conf_str(self, section, data):
    values = data.find("values").findall("value")
    conf_str = "[" + section + "]\n"
    for value in values:
      conf_str+="T" + value.attrib["idx"] + "__" + value.attrib["unit"] + "=" + value.text + "\n"                  
    return conf_str
    
  def run(self):
    self.alive = True
    self.end = False
    #even_odd_flag = 1
    
    while True:
      if self.end:
        self.alive = False
        return
      data = self.get_data()
      if data != "Err":
        self.send_data(data)
        self.save_to_file(data)
      self.zzzzz(self.interval)
    """
    try:
      ser = serial.Serial(self.serial_port, self.baud)
      try:
        while True:
          #a temporary hack!
          if even_odd_flag == 0:
            even_odd_flag = 1
          else:
            even_odd_flag = 0
          if self.end:
            ser.close()
            self.alive = False
            return
          sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
          result = sock.connect_ex((self.vpn_host, self.vpn_port))
          try:
            sock.close()
          except Exception:
            pass
          if result == 0: 
            self.ctucmd(ser, 'vPNON')
          d = {}
          d['dev'] = 'CTU-03';
          d['ts'] = time.strftime('%Y-%m-%d-%H-%M-%S')
          r = self.ctucmd(ser, 'FWVER')
          if len(r) == 1:
            j = r[0].find('firmware ver')
            if j != -1:
              d['fw'] = r[0][j+12:].rstrip()
          r = self.ctucmd(ser, 'SENS')
          for i in range(len(r)):
            s = r[i].split('=')
            if len(s) == 2:
              j = -1
              u = '__?'
              if s[0].startswith('U'):
                j = s[1].find('V')
                u = '__V'
              if s[0].startswith('T'):
                j = s[1].find('C')
                u = '__C'
              if j != -1:
                d[s[0]+u] = s[1][:j]
          try:
            if even_odd_flag == 0:
              line = d['ts'] + "\t" + d['Ub__V'] + "\t" + d['Ti__C']
              for i in range(1, 6):
                line = line + "\t" + d['T' + str(i) + "__C"]
              line = line + "\n"
              fd = open("D:/siwim_mkiii/siwim_e/log/ctu_" +  datetime.datetime.today().strftime('%Y-%m-%d') + ".txt", "a")
              fd.write(line)
              fd.close()
          except Exception, ex:
            print str(ex)
          try:
            for key, mod in self.downstream_modules_dict.iteritems():
              mod.set_ctu_data(d['fw'], d['Ub__V'])
          except Exception, ex:
            pass
          if 'T1__C' in d:
            msg = self.dict2confstr('mydevice', d) #json.dumps(d)
            self.log(msg) #DEBUG
            sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            try:
              sock.sendto(msg, (self.udp_ip, self.udp_port))
            finally:
              sock.close()
          self.zzzzz(self.interval)
      finally:
        ser.close()
    except Exception, ex:
      self.log(str(ex))
      self.alive = False
    """