import socket 
import base64
import hashlib
import threading
import os
import subprocess
import time
 
from lxml import etree
from generic_module import Module
 
class PictureModule(Module):
  def __init__(self, name, port, backlog, recv_buffer, root_path, old = 1, default_cam_type = "mobotix", default_cam_names = list()):
    Module.__init__(self, name, root_path)
    self.port = port
    self.backlog = backlog
    self.recv_buffer = recv_buffer
    
    self.parser = etree.XMLParser(remove_blank_text=True)
    self.sock = None
    self.old = old #a simple flag for this module to know where/how to look for photos. once the old ways are done and forgotten, remove this flag and everything that gave it meaning. 
    
    #todo; add support for that to conf, runner etc.
    self.default_cam_type = default_cam_type #mobotix, technix and so on ...
    self.default_cam_names = default_cam_names #list of actual module names
    
    self.num_of_clients = 0
    self.clients_lock = threading.Lock()
    
    #self.clients_lock = threading.Lock()
    #self.threads = list()
    
  def set_end(self):
    self.end = True
    #this will initiate shutdown procedure; a bit hacky, but this is how we do thing around here
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)  
    s.connect(("127.0.0.1", self.port))
    try:
      s.close()
    except Exception:
      pass
    
  def find_path(self, dt, lane, site, photo_root):
    dt_parts = dt.split("-")
    dt_dy = dt_parts[0] + "-" + dt_parts[1] + "-" + dt_parts[2] + "-hh-mm-ss"
    dt_hr = dt_parts[0] + "-" + dt_parts[1] + "-" + dt_parts[2] + "-" + dt_parts[3] + "-mm-ss"
    partial_path = self.sites_path + "/" + site + "/ext/" + photo_root
    cam_types = list()
    if self.default_cam_type == "":
      cam_types = os.listdir(partial_path)
    else:
      cam_types.append(self.default_cam_type)
    for cam_type in cam_types:
      partial_path = partial_path + "/" + cam_type
      partial_path_dirs = os.listdir(partial_path)
      for partial_path_dir in partial_path_dirs:
        if len(self.default_cam_names) > 0 and partial_path_dir not in self.default_cam_names:
          continue
        path = partial_path + "/" + partial_path_dir + "/" + dt_dy + "/" + dt_hr
        pictures = os.listdir(path)
        for picture in pictures:
          #print dt + "_" + lane + "_00"
          #print picture
          #self.log(picture)
          #self.log(dt + "_" + lane + "_00.jpg")
          if (dt + "_" + lane + "_00.jpg") == picture:
            return path + "/" + picture
    return None

  def md5(self, fname):
    hash_md5 = hashlib.md5()
    with open(fname, "rb") as f:
      for chunk in iter(lambda: f.read(4096), b""):
        hash_md5.update(chunk)
    return hash_md5.hexdigest()
  
  #remove this ASAP. it's an ugly piece of heritage. 
  def old_siwim_picture_getter(self, dt, site):
    fname = self.root_path + "/sites/" + site + "/live/camera/" + dt[0:10] + "-hh-mm-ss/" + dt[0:13] + "-mm-ss/" + dt
    fname1 = self.root_path + "/sites/" + site + "/live/camera1/" + dt[0:10] + "-hh-mm-ss/" + dt[0:13] + "-mm-ss/" + dt
    fname2 = self.root_path + "/sites/" + site + "/live/camera2/" + dt[0:10] + "-hh-mm-ss/" + dt[0:13] + "-mm-ss/" + dt
    the_fname = ""
    if os.path.isfile(fname + ".vehiclephotos"):
      the_fname = fname
    elif os.path.isfile(fname1 + ".vehiclephotos"):
      the_fname = fname1
    elif os.path.isfile(fname2 + ".vehiclephotos"):
      the_fname = fname2 
    else:
      return None, None
    FNULL = open(os.devnull, 'w')
    converting = subprocess.Popen(["vehiclephotos2jpg.exe", the_fname + ".vehiclephotos"], stdout=FNULL, stderr=subprocess.STDOUT)
    converting.communicate()
    out = open(the_fname + ".jpg", "rb")
    binary_data = out.read()
    out.close()
    hsh = self.md5(the_fname + ".jpg")
    os.remove(the_fname + ".jpg")
    return binary_data, hsh
  
  def form_message(self, info):
    top_element = etree.Element("i-message", type = "pic_response")
    for key, val in info.iteritems():
      tag = etree.Element(key)
      tag.text = val
      top_element.append(tag)
    return etree.tostring(top_element, pretty_print=False)    
            
  def clientthread(self, conn):
    if self.end:
      return
    with self.clients_lock:
      self.num_of_clients+=1
    try:
      line = ""
      timeout = 5
      cnt = 0
      starting_tag_id = -1
      closing_tag_id = -1
      while cnt < timeout and (starting_tag_id < 0 or closing_tag_id < 0):
        #with self.clients_lock:
          #self.sock.setblocking(1)
        line = line + conn.recv(self.recv_buffer)
        #print "received"
          #self.sock.settimeout(1.0)
        starting_tag_id = line.find("<i-message ")
        closing_tag_id = line.find("</i-message>")
        cnt+=1
      if starting_tag_id < 0 or closing_tag_id < 0:
        conn.send(self.form_message({"status":"ERR"}))
        conn.close()
        with self.clients_lock:
          self.num_of_clients-=1
        return
      original_xml = None
      try:
        original_xml = etree.fromstring(line, self.parser)  
      except Exception:
        self.log(line)
        conn.send(self.form_message({"status":"ERR"}))
        conn.close()
        with self.clients_lock:
          self.num_of_clients-=1
        return
      i_message = original_xml 
      if i_message.attrib["type"] == "pic_request":
        picture = i_message.find("picture")
        ts = picture.find("ts").text
        lane = picture.find("lane").text
        site_name = picture.find("site_name").text
        photo_type = picture.find("photo_type")
        encoding = "default"
        try:
          encoding = picture.find("encoding").text  
        except Exception, ex:
          pass
        #TODO: picture type?
        photo_root = "photo"
        try:
          pic_type = picture.attrib["type"]
          if pic_type == "anpr":
            photo_root = "anpr"
        except Exception:
          pass
        pic_data = None
        md = None
        if self.old == 1:
          try:
            pic_data, md = self.old_siwim_picture_getter(ts, site_name)
          except Exception:
            conn.send(self.form_message({"status":"ERR"}))
            conn.close()
            with self.clients_lock:
              self.num_of_clients-=1
            return
        else:
          #photo_root = "photo"
          #if photo_type != None and photo_type.text == "anpr":
            #photo_root = "anpr"
          path = self.find_path(ts, lane, site_name, photo_root)
          #print path
          try:
            fd = open(path, "rb")
            pic_data = fd.read()
            fd.close()
          except Exception, ex:
            self.log(str(ex))
            conn.send(self.form_message({"status":"ERR"}))
            conn.close()
            with self.clients_lock:
              self.num_of_clients-=1
            return
          md = str(self.md5(path))
        #print pic_data
        #print md
        if pic_data != None and md != None:
          dct = {"status":"OK", "size":str(len(pic_data)), "format":"jpg", "md5":md, "encoding":encoding}
          if encoding == "base64":
            dct["picture"] = base64.b64encode(pic_data)
            conn.send(self.form_message(dct))
            conn.close()
            with self.clients_lock:
              self.num_of_clients-=1
            return
          elif encoding == "default" or encoding == "binary":
            conn.send(self.form_message(dct))
            conn.send(pic_data)
            conn.close()
            with self.clients_lock:
              self.num_of_clients-=1
            return
        else:
          conn.send(self.form_message({"status":"ERR"}))
          conn.close()
          with self.clients_lock:
            self.num_of_clients-=1
          return
      else:
        conn.send(self.form_message({"status":"ERR"}))
        conn.close()
        with self.clients_lock:
          self.num_of_clients-=1
        return 
    except Exception, ex:
      self.log(str(ex))
      conn.send(self.form_message({"status":"ERR"}))
      conn.close()
      with self.clients_lock:
        self.num_of_clients-=1
      return
  
  #def count_alives(self):
    #cnt = 0
    #for thr in self.threads:
      #if thr.is_alive():
        #cnt+=1
    #return cnt
        
  def run(self):
    self.alive = True
    self.end = False
    try:
      self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM) 
      self.sock.bind(('', self.port)) 
      self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
      self.sock.listen(self.backlog)
    except Exception, ex:
      self.log(str(ex))
      self.alive = False
      return
    try:
      while  True:
        conn, addr = self.sock.accept()
        if self.end:
          while True:
            with self.clients_lock:
              if self.num_of_clients == 0:
                try:
                  conn.close()
                except Exception:
                  pass
                self.sock.close()
                self.alive = False
                return
            self.zzzzz(0.5)
        threading.Thread(target=self.clientthread, args=(conn,)).start()
    except Exception, ex:
      self.log(str(ex))
      self.sock.close()
    self.alive = False
      