import time
import socket
import datetime
import os

from lxml import etree
from generic_module import Module

class ReceiveModule(Module):
  def __init__(self, name, typ, host, port, recv_buffer_size, save_original, root_path, offsets_dict):
    Module.__init__(self, name, root_path)
    self.typ = typ
    self.host = host
    self.port = port
    self.recv_buffer_size = recv_buffer_size
    self.save_original = save_original
    self.offsets_dict = offsets_dict
    
    self.s = None
    self.parser = etree.XMLParser(remove_blank_text=True)
      
  #def calculate_distance(self, s1, ts1, ts2, whlbse1, prepend2, append1):  
    #delta_t = datetime_object_2 - datetime_object_1
    #d_t_seconds = delta_t.total_seconds()
    #s1_fl = float(s1)
    #return d_t_seconds * s1_fl - float(whlbse1) - float(prepend2) - float(append1) #delta time * speed of the first vehicle ~ distance between the two vehicles when the second vehicle is at MP
      
  #def md5(self, fname):
    #hash_md5 = hashlib.md5()
    #with open(fname, "rb") as f:
      #for chunk in iter(lambda: f.read(4096), b""):
        #hash_md5.update(chunk)
    #return hash_md5.hexdigest()
  
  def set_end(self):
    self.end = True
    try:
      self.s.shutdown(socket.SHUT_WR)
    except Exception:
      pass    
  
  def generate_path(self, mod):
    pth = self.sites_path
    pth = pth + "/" + self.sname + "/ext"
    if not os.path.exists(pth):
      os.mkdir(pth)
    pth = pth + "/" + mod
    if not os.path.exists(pth):
      os.mkdir(pth)
    pth = pth + "/" + self.typ
    if not os.path.exists(pth):
      os.mkdir(pth)
    pth = pth + "/" + self.name 
    if not os.path.exists(pth):
      os.mkdir(pth)
    return pth
           
  def save_to_file(self, data_type, str_vehicles):
    if self.sname == "":
      return
    first_line = "<swd><site><name>" + self.sname + "</name><vehicles>\n"
    last_line = "</vehicles></site></swd>"
    try:
      date = datetime.datetime.now().strftime('%Y-%m-%d')
      path = self.generate_path(data_type)
      fd = open(path + "/" + date + ".xml", "a+")
      #remove last line          
      fd.seek(0, os.SEEK_END)
      pos = fd.tell() - 1
      while pos > 0 and fd.read(1) != "\n":
        pos -= 1
        fd.seek(pos, os.SEEK_SET)
      if pos > 0:
        fd.seek(pos, os.SEEK_SET)
        fd.truncate()
      vhs = ""
      for str_vehicle in str_vehicles:
        vhs = vhs + str_vehicle + "\n\n"
      #if file is empty, add first line
      if pos == fd.tell() - 1:
        fd.write(first_line + vhs + last_line)
      else:
        fd.write(vhs + last_line)
      fd.close()
    except Exception, ex:
      try:
        fd.close()
      except Exception:
        pass
      self.log(str(ex))
      
  def connect(self):
    self.s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)    
    try:
      self.s.connect((self.host, self.port))  
      self.s.settimeout(2)
      return 0
    except Exception, ex:
      self.log("Connection (" + self.name + ") failed with the following msg: " + str(ex)) 
      return 1
    
  def siwim_new_xml_receiver(self):
    try:
      for key, mod in self.downstream_modules_dict.iteritems():
        mod.tell_offset(key, self.offsets_dict)
      line = ""
      cnt = 0 #for exiting thread
      while True:
        if self.end:
          return
        received = ""
        try:
          received = self.s.recv(self.recv_buffer_size)
        except socket.timeout:
          continue
        except socket.error, e:
          self.log(str(e))
          return
        else:
          line = line + received
          line = line.replace('<?xml version="1.0" ?>', "")
          starting_tag_id = line.find("<swd ")
          closing_tag_id = line.find("</swd>")
          original_xml = None
          if starting_tag_id != -1 and closing_tag_id != -1:
            dt_str = line[starting_tag_id:closing_tag_id + 6]
            try:
              original_xml = etree.fromstring(dt_str, self.parser)
              #save everything that we didn't use for xml 
              line = line[(closing_tag_id + 6):len(line)]
              cnt = 0
            except Exception:
              self.log("Bogus content, clearing the line.")
              #if data between starting and closing swd tag is "corba", we need to clear the line
              line = ""
              continue  
          else:
            #if there was no starting tag inside line, we need to clear the line
            if starting_tag_id == -1:
              line = ""
            cnt+=1
            if cnt == 8:
              self.log("Exiting thread: " + self.name)
              return
            continue
          self.sname = list(original_xml.iter("name"))[0].text
          original_vehicles = list(original_xml.iter("vehicle"))
          str_vehicles = list()
          for original_vehicle in original_vehicles:
            str_vehicles.append(etree.tostring(original_vehicle))
            for key, mod in self.downstream_modules_dict.iteritems():
              mod.set_site_name(self.sname)
              mod.add_vehicle(original_vehicle, self.name)
          if self.save_original == 1:          
            self.save_to_file("wim", str_vehicles)
    except Exception, ex:
      self.log("Exiting thread: " + self.name + ", " + str(ex))
      return
  
  def comark_lwh_receiver(self): 
    try:
      for key, mod in self.downstream_modules_dict.iteritems():
        mod.tell_offset(key, self.offsets_dict)
        #mod.set_site_name(self.sname)
        #mod.add_vehicle(vehicle, self.name)
      line = ""
      cnt = 0 #for exiting thread
      while True:
        if self.end:
          return
        received = ""
        try:
          received = self.s.recv(self.recv_buffer_size)
        except socket.timeout:
          continue
        except socket.error, e:
          self.log(str(e))
          return
        else:
          line = line + received
          starting_tag_id = line.find("<sensor ")
          closing_tag_id = line.find("</sensor>")
          original_xml = None
          if starting_tag_id != -1 and closing_tag_id != -1:
            dt_str = line[starting_tag_id:closing_tag_id + 9]
            try:
              original_xml = etree.fromstring(dt_str, self.parser)
              #save everything that we didn't use for xml 
              line = line[(closing_tag_id + 9):len(line)]
              cnt = 0
            except Exception, ex:
              #if data between starting and closing swd tag is "corba", we need to clear the line
              self.log("Bogus content, clearing the line.")
              #print line
              line = ""
              continue  
          else:
            #if there was no starting tag inside line, we need to clear the line
            if starting_tag_id == -1:
              #self.log("No starting tag, clearing the line.")
              line = ""
            cnt+=1
            if cnt == 8:
              self.log("Exiting thread: " + self.name)
              return
            #if closing_tag_id == -1:
              #self.log("No closing tag. Continuoing.")
            continue
          #add a container element in case line contains more than one message
          messages = list(original_xml.iter("sensor"))    
          str_vehicles = list()
          for message in messages:
            vehicle = etree.Element("vehicle")
            vehicle_1 = etree.Element("vehicle")
            lwh = etree.Element("lwh")
            vehicle.append(lwh)
            body = list(message.iter("transit_end"))[0]
            attributes = body.attrib
            for key in attributes.keys():
              if key == "time_iso_ms":
                ts = datetime.datetime.strptime(attributes[key], '%Y-%m-%dT%H:%M:%S.%f').strftime('%Y-%m-%d-%H-%M-%S-%f')
                lwh_element = etree.Element("ts")
                lwh_element.text = ts
                lwh.append(lwh_element)
              elif key == "speed":
                lwh_element = etree.Element("v")
                lwh_element.text = attributes[key]
                lwh.append(lwh_element)
              elif key == "length":
                lwh_element = etree.Element(key)
                lwh_element.text = attributes[key]
                lwh.append(lwh_element)
                lwh_element1 = etree.Element("siwim_i_add_to_offset")
                lwh_element1.text = str(float(attributes[key]) / 1000.0)
                lwh.append(lwh_element1)
              else:
                lwh_element = etree.Element(key)
                lwh_element.text = attributes[key]
                lwh.append(lwh_element)
              #vehicle = etree.Element("vehicle")
              vehicle.append(lwh)
              vehicle_1.append(body)
              for key, mod in self.downstream_modules_dict.iteritems():
                mod.set_site_name(self.sname)
                mod.add_vehicle(vehicle, self.name)
            str_vehicles.append(etree.tostring(vehicle_1))
              #self.log(etree.tostring(vehicle_1))
          if self.save_original == 1 and len(str_vehicles) > 0:   
            self.save_to_file("lwh", str_vehicles)
    except Exception, ex:
      self.log(str(ex))
      return          
  
  def technix_anpr_receiver(self): 
    try:
      for key, mod in self.downstream_modules_dict.iteritems():
        mod.tell_offset(key, self.offsets_dict)
      line = ""
      cnt = 0 #for exiting thread
      while True:
        if self.end:
          return
        received = ""
        try:
          received = self.s.recv(self.recv_buffer_size)
        except socket.timeout:
          continue
        except socket.error, e:
          self.log(str(e))
          return
        else:
          line = line + received
          starting_tag_id = line.find("<TAMPxml ")
          closing_tag_id = line.find("</TAMPxml>")
          original_xml = None
          if starting_tag_id != -1 and closing_tag_id != -1:
            dt_str = line[starting_tag_id:closing_tag_id + 10]
            try:
              original_xml = etree.fromstring(dt_str, self.parser)
              #save everything that we didn't use for xml 
              line = line[(closing_tag_id + 10):len(line)]
              cnt = 0
            except Exception, ex:
              #if data between starting and closing swd tag is "corba", we need to clear the line
              self.log("Bogus content, clearing the line.")
              #print line
              line = ""
              continue  
          else:
            #if there was no starting tag inside line, we need to clear the line
            if starting_tag_id == -1:
              #self.log("No starting tag, clearing the line.")
              line = ""
            cnt+=1
            if cnt == 8:
              self.log("Exiting thread: " + self.name)
              return
            #if closing_tag_id == -1:
              #self.log("No closing tag. Continuoing.")
            continue
          #add a container element in case line contains more than one message
          messages = list(original_xml.iter("Message"))    
          str_vehicles = list()
          for message in messages:
            if message.attrib["Type"] == "KeepAlive":
              top_element = etree.Element("TAMPxml", LocationId = "TAMPxmlClient")
              ka_message = etree.Element("Message", Type = "KeepAlive")
              top_element.append(ka_message)
              self.s.send(etree.tostring(top_element))
            elif message.attrib["Type"] == "ANPRevent":
              vehicle = etree.Element("vehicle")
              vehicle_1 = etree.Element("vehicle")
              anpr = etree.Element("anpr")
              vehicle.append(anpr)
              body = list(message.iter("Body"))[0]
              attributes = body.attrib
              kn_num = ""
              un_num = ""
              for key in attributes.keys():
                if key == "TRIGGER_TIME":
                  first_split = attributes[key].split(" ")
                  date_split = first_split[0].split(".")
                  time_split_1 = first_split[1].split(".")
                  time_split_2 = time_split_1[0].split(":")
                  ts = date_split[2] + "-" + date_split[1] + "-" + date_split[0] + "-" + time_split_2[0] + "-" + time_split_2[1] + "-" + time_split_2[2] + "-" + time_split_1[1]
                  anpr_element = etree.Element("ts")
                  anpr_element.text = ts
                  anpr.append(anpr_element)
                  continue
                elif key == "KN_NUM":
                  kn_num = attributes[key]
                elif key == "UN_NUM":
                  un_num = attributes[key]
                elif key == "ANPR_TXT":
                  lp_el = etree.Element("lp")
                  lp_el.text = attributes[key]
                  anpr.append(lp_el)
                elif key == "ANPR_IMAGE_PATH":
                  anpr_ip_el = etree.Element("anpr_ip")
                  anpr_ip_el.text = attributes[key]
                  anpr.append(anpr_ip_el)
                elif key == "ADR_IMAGE_PATH":
                  adr_ip_el = etree.Element("adr_ip")
                  adr_ip_el.text = attributes[key]
                  anpr.append(adr_ip_el)
                elif key == "COMBINED_TRG_IMAGE_PATH":
                  trg_ip_el = etree.Element("trg_ip")
                  trg_ip_el.text = attributes[key]
                  anpr.append(trg_ip_el)
                elif key == "LANE_ID":
                  lane_el = etree.Element("lane")
                  #ugly hack! just take the last character, due to a possibility of lane text actually being "Lane1"
                  #could be something else, but this should cover most cases
                  lane_el.text = attributes[key][len(attributes[key]) - 1]
                  anpr.append(lane_el)
              hdp_el = etree.Element("hgp")
              hdp_el.text = kn_num + " " + un_num
              anpr.append(hdp_el)
              #vehicle = etree.Element("vehicle")
              vehicle.append(anpr)
              vehicle_1.append(body)
              for key, mod in self.downstream_modules_dict.iteritems():
                mod.set_site_name(self.sname)
                mod.add_vehicle(vehicle, self.name)
              str_vehicles.append(etree.tostring(vehicle_1))
          if self.save_original == 1 and len(str_vehicles) > 0:   
            self.save_to_file("anpr", str_vehicles)
    except Exception, ex:
      self.log(str(ex))
      return
     
  def run(self):
    self.alive = True
    self.end = False
    if self.connect() == 1:
      self.log("Connection failed.")
    else:
      for key, mod in self.downstream_modules_dict.iteritems():
        mod.register_alarm(self.name)
      if self.typ == "technix":
        self.technix_anpr_receiver()
      elif self.typ == "cestel":
        self.siwim_new_xml_receiver()
      elif self.typ == "macq":
        self.macq_anpr_receiver()
      elif self.typ == "comark":
        self.comark_lwh_receiver()
    self.alive = False
    try:
      self.s.close()
    except Exception:
      pass
