import datetime
import sys
from sys import platform
import urllib
import socket 
import re
import array
import httplib
import ConfigParser
import os
import ssl
import subprocess

if platform == "win32":
  import _winreg
  from ctypes import windll, byref, c_ulonglong, c_wchar_p, pointer, create_string_buffer, c_uint, string_at
  #from ctypes.wintypes import DWORD
#elif platform == "linux" or platform == "linux2":
  
  #pass

from datetime import timedelta
from lxml import etree
from generic_module import Module
 
class StatusModule(Module):
  def __init__(self, name, host, time_interval, version, url, protocol, root_path, ntp_conf_path, ovpn_path, save_curr_to_disk):
    Module.__init__(self, name, root_path)
    self.time_interval = time_interval
    self.host = host
    self.version = version
    self.url = url
    self.protocol = protocol 
    self.ntp_conf_path = ntp_conf_path
    self.ovpn_path = ovpn_path
    self.save_curr_to_disk = save_curr_to_disk
    
    self.ctu_fw_version = ""
    self.hist_fw_version = ""
    self.voltage = ""
    
    self.mcp_mod_time = 0
    self.mcp_version = ""
    
  def set_ctu_data(self, fw_version, voltage):
    self.ctu_fw_version = fw_version
    self.voltage = voltage
    
  def set_hist_data(self, fw_version, voltage):
    self.hist_fw_version = fw_version
    self.voltage = voltage

  def swmCurSite(self, loc):
    try:
      cfg = ConfigParser.RawConfigParser()
      cfg.read(loc)
      #cfg.read('d:/siwim_mkiii/siwim_e/conf/siwim.conf')
      return cfg.get('global', 'site')
    except:
      return 'unknown'
    
  def w32ovpn_version(self):  
    sp = subprocess.Popen([self.ovpn_path, "--version"], stdout=subprocess.PIPE)
    output = sp.communicate()[0]
    return output.split(" ")[1]
  """
  def uniX32ovpn_version(self):  
    sp = subprocess.Popen(["dpkg", "-l", "openvpn"], stdout=subprocess.PIPE)
    output = sp.communicate()[0]
    lines = output.split("\n")
    #self.log(str(lines))
    ver = lines[len(lines) - 2].split()[2]
    return ver
  """  
  #def uniXswmCurSite(self):
    #try:
      #cfg = C  onfigParser.RawConfigParser()
      #cfg.read('/siwim/siwim_e/conf/siwim.conf')
      #return cfg.get('global', 'site')
    #except:
      #return 'unknown'
            
  def w32CapacityMB(self, p):
    lpFreeBytesAvailable = c_ulonglong(0)
    lpTotalNumberOfBytes = c_ulonglong(0)
    lpTotalNumberOfFreeBytes = c_ulonglong(0)
    windll.kernel32.GetDiskFreeSpaceExW(c_wchar_p(p), pointer(lpFreeBytesAvailable), pointer(lpTotalNumberOfBytes), pointer(lpTotalNumberOfFreeBytes))
    return lpTotalNumberOfBytes.value / 1024 / 1024
    
  def uniX32CapacityMB(self, mp):
    sp = subprocess.Popen(["df", "-B", "MB", mp], stdout=subprocess.PIPE)
    output = sp.communicate()[0]
    size = output.split("\n")[1].split()[1]
    return int(size[:size.find("M")])
  
  def getUniXhostname(self):
    sp = subprocess.Popen(["hostname"], stdout=subprocess.PIPE)
    output = sp.communicate()[0]
    return output.strip()

  def w32get_file_info(self, filename, info):
    size = windll.version.GetFileVersionInfoSizeA(filename, None)
    if size == 0:
      return ''
    res = create_string_buffer(size)
    windll.version.GetFileVersionInfoA(filename, None, size, res)
    r = c_uint()
    l = c_uint()
    windll.version.VerQueryValueA(res, '\\VarFileInfo\\Translation', byref(r), byref(l))
    if not l.value:
      return '' 
    codepages = array.array('H', string_at(r.value, l.value))
    codepage = tuple(codepages[:2].tolist())
    windll.version.VerQueryValueA(res, ('\\StringFileInfo\\%04x%04x\\'+ info) % codepage, byref(r), byref(l))
    return string_at(r.value) # (r.value, l.value)
  
  def uniXget_file_info(self):
    if self.mcp_mod_time != os.path.getmtime(self.root_path + "/siwim_e/bin/siwim_mcp"):
      sp = subprocess.Popen([self.root_path + "/siwim_e/bin/siwim_mcp", "-h"], stdout=subprocess.PIPE)
      output = sp.communicate()[0]
      relevant_line = output.split("\n")[1]
      self.mcp_version = relevant_line[relevant_line.find("v") + 1:]
      self.mcp_mod_time = os.path.getmtime(self.root_path + "/siwim_e/bin/siwim_mcp")
    return self.mcp_version

  def get_registry_value(self, key, subkey, value):
    key = getattr(_winreg, key)
    handle = _winreg.OpenKey(key, subkey)
    (value, type) = _winreg.QueryValueEx(handle, value)
    return value

  def _cpu(self):
    try:
      return self.get_registry_value("HKEY_LOCAL_MACHINE", "HARDWARE\\DESCRIPTION\\System\\CentralProcessor\\0", "ProcessorNameString")
    except:
      return self.get_registry_value("HKEY_LOCAL_MACHINE", "HARDWARE\\DESCRIPTION\\System\\CentralProcessor\\0", "Identifier")

  def _mhz(self):
    return self.get_registry_value("HKEY_LOCAL_MACHINE", "HARDWARE\\DESCRIPTION\\System\\CentralProcessor\\0", "~MHz")

  def _sysver(self):
    return 'MK3'

  #class MEMORYSTATUS(Structure):
    #_fields_ = [('dwLength', DWORD), ('dwMemoryLoad', DWORD), ('dwTotalPhys', DWORD), ('dwAvailPhys', DWORD), ('dwTotalPageFile', DWORD), ('dwAvailPageFile', DWORD), ('dwTotalVirtual', DWORD), ('dwAvailVirtual', DWORD),]

  #def winmem(self):
    #ms = self.MEMORYSTATUS()
    #windll.kernel32.GlobalMemoryStatus(byref(ms))
    #return ms

  def gpsLoc(self):
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.settimeout(5.0)
    data = 'No GPS data'
    try:
      s.connect(('192.168.4.100', 9494))
      data = s.recv(1024)
      s.close()
    except:
      pass

    p = re.compile('GPGGA,([.\d]*),([.\d]*),([NS]),([.\d]*),([EW])')
    a = p.search(data)

    if a == None:
      return (0, 0)
    else:
      latd = float(a.group(2)[:2])
      lond = float(a.group(4)[:3])
      latm = float(a.group(2)[2:])
      lonm = float(a.group(4)[3:])
      
      lat = latd + (latm / 60)
      lon = lond + (lonm / 60)

      if a.group(3) != 'N':
        lat = -lat
      if a.group(5) != 'E':
        lon = -lon
      return (lat, lon)

  def _os_version(self):
    os = self.get_registry_value("HKEY_LOCAL_MACHINE","SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion","ProductName")
    try:
      sp = self.get_registry_value("HKEY_LOCAL_MACHINE","SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion","CSDVersion")
    except:
      sp = 'Vanilla'
    build = self.get_registry_value("HKEY_LOCAL_MACHINE","SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion","CurrentBuildNumber")
    return "%s %s (build %s)" % (os, sp, build)

  def _recvfind(self, sock, n, to_sec=5):
    d = ''
    absto = datetime.datetime.now() + timedelta(seconds=to_sec)
    while (-1 == d.find(n)) and (datetime.datetime.now() < absto):
      d += sock.recv(256)
    return re.sub(r'[^a-zA-Z0-9 \n]', '', d)

  def SierraImsi(self, pwd):
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.settimeout(5.0);
    imsi = 'No IMSI'
    try:
      s.connect(('192.168.4.100', 2332))
      data = self._recvfind(s, ':')
      if data.find('login') >= 0:
        #print 'GX'
        s.sendall('user\r\n')
        data = self._recvfind(s, ':')
        #print 'MP'
      s.sendall(pwd + '\r\n')
      data = self._recvfind(s, 'OK')
      s.sendall('at+cimi?\r\n')
      data = self._recvfind(s, 'OK')
      data = data.split()
      if len(data) > 1: imsi = data[-2]
      s.close()
    except Exception:
      pass 
    return imsi

  def swmiver(self):
    ver = str(self.version[0]) + "." + str(self.version[1])
    return ver

  def ntpsvr(self, loc):
    fn = loc
    if not os.path.exists(fn):
      return 'missing'
    f = open(fn)
    try:
      lines = f.readlines()
    finally:
      f.close()
    for l in lines:
      if l.strip().startswith('server'):
        return l.strip().split()[1] 
    for l in lines:
      if l.strip().startswith('pool'):
        return l.strip().split()[1] 
    return 'unknown'
  
  def w32DiskFreeMB(self, p):
    lpFreeBytesAvailable = c_ulonglong(0)
    lpTotalNumberOfBytes = c_ulonglong(0)
    lpTotalNumberOfFreeBytes = c_ulonglong(0)
    windll.kernel32.GetDiskFreeSpaceExW(c_wchar_p(p), pointer(lpFreeBytesAvailable), pointer(lpTotalNumberOfBytes), pointer(lpTotalNumberOfFreeBytes))
    return lpTotalNumberOfFreeBytes.value / 1024 / 1024
  
  def uniX32PartitionFreeMB(self, mp):
    sp = subprocess.Popen(["df", "-B", "MB", mp], stdout=subprocess.PIPE)
    output = sp.communicate()[0]
    size = output.split("\n")[1].split()[3]
    return int(size[:size.find("M")])

  def swmstat(self):
    kvp = {}
    try:
      kvp['python_version']      = str(sys.version_info[0])+'.'+str(sys.version_info[1])+'.'+str(sys.version_info[2])
    except Exception, ex:
      self.log(str(ex))
    try:
      kvp['timestamp']  = datetime.datetime.now().strftime('%Y-%m-%d %H:%M')  
    except Exception, ex:
      self.log(str(ex))
    try:
      if platform == "win32":
        kvp['computer_name']   = os.getenv('COMPUTERNAME')
      else:
        kvp['computer_name']   = self.getUniXhostname()
    except Exception, ex:
      self.log(str(ex))
    try:
      if platform == "win32":
        kvp['disk_c_free'] = self.w32DiskFreeMB('c:')
      else:
        kvp['disk_c_free'] = self.uniX32PartitionFreeMB('/')
    except Exception, ex:
      self.log(str(ex))
    try:
      if platform == "win32":
        kvp['disk_d_free'] = self.w32DiskFreeMB('d:')
      else:
        kvp['disk_d_free'] = self.uniX32PartitionFreeMB(self.root_path)
    except Exception, ex:
      self.log(str(ex))
    try:
      if platform == "win32":
        kvp['disk_e_free'] = self.w32DiskFreeMB('e:')
      else:
        kvp['disk_e_free'] = self.uniX32PartitionFreeMB('/')
    except Exception, ex:
      self.log(str(ex))
    try:
      if platform == "win32":
        kvp['siwim_site_name']       = self.swmCurSite(self.root_path + "/siwim_e/conf/siwim.conf").decode('iso-8859-1').encode('ascii','replace')
      else:
        kvp['siwim_site_name']       = self.swmCurSite(self.root_path + "/siwim_e/conf/siwim.conf").decode('iso-8859-1').encode('ascii','replace')
    except Exception, ex:
      self.log(str(ex))
    try:
      kvp['siwim_system_voltage']       = self.swmVoltageCTU()
    except Exception, ex:
      self.log(str(ex))
    try:
      if platform == "win32":
        kvp['siwim_e_version']    = self.w32get_file_info(self.root_path + '/siwim_e/bin/siwim_mcp.exe', 'FileVersion')
      else:
        kvp['siwim_e_version']    = self.uniXget_file_info()
    except Exception, ex:
      self.log(str(ex))
    try:
      kvp['siwim_i_version']    = self.swmiver()
    except Exception, ex:
      self.log(str(ex))
    try:
      (lat,lon) = self.gpsLoc()
      kvp['gps_lat']        = lat
      kvp['gps_lon']        = lon
    except Exception, ex:
      self.log(str(ex))
    try:
      if platform == "win32":
        kvp['disk_c_total'] = self.w32CapacityMB('c:')
      else:
        kvp['disk_c_total'] = self.uniX32CapacityMB('/')
    except Exception, ex:
      self.log(str(ex))
    try:
      if platform == "win32":
        kvp['disk_d_total'] = self.w32CapacityMB('d:')
      else:
        kvp['disk_d_total'] = self.uniX32CapacityMB(self.root_path)
    except Exception, ex:
      self.log(str(ex))
    try:
      if platform == "win32":
        kvp['disk_e_total'] = self.w32CapacityMB('e:')
      else:
        kvp['disk_e_total'] = self.uniX32CapacityMB('/')
    except Exception, ex:
      self.log(str(ex))
    try:
      kvp['gsm_imsi']       = self.SierraImsi("12345")
      if kvp['gsm_imsi'] == 'No IMSI':
        kvp['gsm_imsi']       = self.SierraImsi("siwim323")
    except Exception, ex: 
      self.log(str(ex))
    try:
      if self.ntp_conf_path == "":
        if platform == "win32":
          kvp['ntp_server']        = self.ntpsvr("c:/windows/system32/drivers/etc/ntp.conf") 
        else:
          kvp['ntp_server']        = self.ntpsvr("/etc/ntp.conf") 
      else:
        kvp['ntp_server']        = self.ntpsvr(self.ntp_conf_path) 
    except Exception, ex:
      self.log(str(ex))
    try:
      if self.ctu_fw_version != "":
        kvp['ctu_firmware_version']        = self.ctu_fw_version
    except Exception, ex:
      self.log(str(ex))
    try:
      if self.hist_fw_version != "":
        kvp['hist_firmware_version']        = self.hist_fw_version
    except Exception, ex:
      self.log(str(ex))
    try:
      #if platform == "win32":
      kvp['openvpn_version']     = self.w32ovpn_version()
      #else:
        #kvp['openvpn_version']     = self.uniX32ovpn_version()
    except Exception, ex:
      self.log(str(ex))
    return kvp
    
  def swmVoltageCTU(self):
    if self.voltage != "":
      return str(self.voltage)
    else:
      fn = self.root_path + '/siwim_e/log/ctu_' + datetime.datetime.now().strftime('%Y-%m-%d') + '.txt'
      if not os.path.exists(fn):
        return '?'
      try:
        f = open(fn,'r')
        return f.readlines()[-1].split('\t')[1][:4]
      finally:
        f.close()
    
  def run(self):
    self.alive = True
    self.end = False
    try:
      while True:
        if self.end:
          self.alive = False
          return
        try:
          kvp = self.swmstat()
          root = etree.Element("swmstat") 
          for key, val in kvp.iteritems():
            node = etree.Element(key) 
            node.text = str(val)
            root.append(node)
          if self.save_curr_to_disk == 1:
            self.log("Writing status ...")
            fd = open("log/swmstat.xml", "w")
            fd.write(etree.tostring(root))
            fd.close()
          payload = urllib.urlencode({'sys':kvp["computer_name"].lower(), 'msg':etree.tostring(root)})
          headers = {"Content-type": "application/x-www-form-urlencoded", "Accept": "text/plain"}
          if self.protocol == "https":
            conn = httplib.HTTPSConnection(self.host, context=ssl._create_unverified_context(), timeout=10)
          elif self.protocol == "http":
            conn = httplib.HTTPConnection(self.host, timeout=10)
          #print self.host + self.url
          conn.request('POST', self.url, payload, headers)
          resp = conn.getresponse()
          self.log(resp.read())
        except Exception, ex:
          self.log(str(ex))
        #time.sleep(self.time_interval)
        self.zzzzz(self.time_interval)
    except Exception, ex:
      self.log(str(ex))
      self.alive = False
      